@@ -19,19 +19,6 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-import {
-    TRUSTED_DIRECTIVE_BASE_RULE_ID,
-    getDynamicRules,
-} from './ruleset-manager.js';
-
-import {
-    adminRead,
-    browser,
-    dnr,
-    localRead, localRemove, localWrite,
-    sessionRead, sessionWrite,
-} from './ext.js';
-
 import {
     broadcastMessage,
     hostnamesFromMatches,
@@ -39,6 +26,15 @@ import {
     toBroaderHostname,
 } from './utils.js';
 
+import {
+    browser,
+    localRead, localWrite,
+    sessionRead, sessionWrite,
+} from './ext.js';
+
+import { adminReadEx } from './admin.js';
+import { filteringModesToDNR } from './ruleset-manager.js';
+
 /******************************************************************************/
 
 // 0:       no filtering
@@ -73,19 +69,6 @@ const pruneHostnameFromSet = (hostname, hnSet) => {
 
 /******************************************************************************/
 
-const eqSets = (setBefore, setAfter) => {
-    if ( setBefore.size !== setAfter.size ) { return false; }
-    for ( const hn of setAfter ) {
-        if ( setBefore.has(hn) === false ) { return false; }
-    }
-    for ( const hn of setBefore ) {
-        if ( setAfter.has(hn) === false ) { return false; }
-    }
-    return true;
-};
-
-/******************************************************************************/
-
 const serializeModeDetails = details => {
     return {
         none: Array.from(details.none),
@@ -224,38 +207,40 @@ function applyFilteringMode(filteringModes, hostname, afterLevel) {
 
 /******************************************************************************/
 
-async function readFilteringModeDetails() {
-    if ( readFilteringModeDetails.cache ) {
-        return readFilteringModeDetails.cache;
-    }
-    const sessionModes = await sessionRead('filteringModeDetails');
-    if ( sessionModes instanceof Object ) {
-        readFilteringModeDetails.cache = unserializeModeDetails(sessionModes);
-        return readFilteringModeDetails.cache;
+export async function readFilteringModeDetails(bypassCache = false) {
+    if ( bypassCache === false ) {
+        if ( readFilteringModeDetails.cache ) {
+            return readFilteringModeDetails.cache;
+        }
+        const sessionModes = await sessionRead('filteringModeDetails');
+        if ( sessionModes instanceof Object ) {
+            readFilteringModeDetails.cache = unserializeModeDetails(sessionModes);
+            return readFilteringModeDetails.cache;
+        }
     }
     let [ userModes, adminNoFiltering ] = await Promise.all([
         localRead('filteringModeDetails'),
-        localRead('adminNoFiltering'),
+        adminReadEx('noFiltering'),
     ]);
     if ( userModes === undefined ) {
         userModes = { basic: [ 'all-urls' ] };
     }
     userModes = unserializeModeDetails(userModes);
     if ( Array.isArray(adminNoFiltering) ) {
+        if ( adminNoFiltering.includes('-*') ) {
+            userModes.none.clear();
+        }
         for ( const hn of adminNoFiltering ) {
-            applyFilteringMode(userModes, hn, 0);
+            if ( hn.charAt(0) === '-' ) {
+                userModes.none.delete(hn.slice(1));
+            } else {
+                applyFilteringMode(userModes, hn, 0);
+            }
         }
     }
     filteringModesToDNR(userModes);
     sessionWrite('filteringModeDetails', serializeModeDetails(userModes));
     readFilteringModeDetails.cache = userModes;
-    adminRead('noFiltering').then(results => {
-        if ( results ) {
-            localWrite('adminNoFiltering', results);
-        } else {
-            localRemove('adminNoFiltering');
-        }
-    });
     return userModes;
 }
 
@@ -267,10 +252,11 @@ async function writeFilteringModeDetails(afterDetails) {
     localWrite('filteringModeDetails', data);
     sessionWrite('filteringModeDetails', data);
     readFilteringModeDetails.cache = unserializeModeDetails(data);
-
-    Promise.all([
+    return Promise.all([
         getDefaultFilteringMode(),
         getTrustedSites(),
+        localWrite('filteringModeDetails', data),
+        sessionWrite('filteringModeDetails', data),
     ]).then(results => {
         broadcastMessage({
             defaultFilteringMode: results[0],
@@ -281,93 +267,6 @@ async function writeFilteringModeDetails(afterDetails) {
 
 /******************************************************************************/
 
-async function filteringModesToDNR(modes) {
-    const dynamicRuleMap = await getDynamicRules();
-    const trustedRule = dynamicRuleMap.get(TRUSTED_DIRECTIVE_BASE_RULE_ID+0);
-    const beforeRequestDomainSet = new Set(trustedRule?.condition.requestDomains);
-    const beforeExcludedRrequestDomainSet = new Set(trustedRule?.condition.excludedRequestDomains);
-    if ( trustedRule !== undefined && beforeRequestDomainSet.size === 0 ) {
-        beforeRequestDomainSet.add('all-urls');
-    } else {
-        beforeExcludedRrequestDomainSet.add('all-urls');
-    }
-
-    const noneHostnames = new Set([ ...modes.none ]);
-    const notNoneHostnames = new Set([ ...modes.basic, ...modes.optimal, ...modes.complete ]);
-    let afterRequestDomainSet = new Set();
-    let afterExcludedRequestDomainSet = new Set();
-    if ( noneHostnames.has('all-urls') ) {
-        afterRequestDomainSet = new Set([ 'all-urls' ]);
-        afterExcludedRequestDomainSet = notNoneHostnames;
-    } else {
-        afterRequestDomainSet = noneHostnames;
-        afterExcludedRequestDomainSet = new Set([ 'all-urls' ]);
-    }
-
-    if ( eqSets(beforeRequestDomainSet, afterRequestDomainSet) ) {
-        if ( eqSets(beforeExcludedRrequestDomainSet, afterExcludedRequestDomainSet) ) {
-            return;
-        }
-    }
-
-    const removeRuleIds = [
-        TRUSTED_DIRECTIVE_BASE_RULE_ID+0,
-        TRUSTED_DIRECTIVE_BASE_RULE_ID+1,
-    ];
-    dynamicRuleMap.delete(TRUSTED_DIRECTIVE_BASE_RULE_ID+0);
-    dynamicRuleMap.delete(TRUSTED_DIRECTIVE_BASE_RULE_ID+1);
-
-    const allowEverywhere = afterRequestDomainSet.delete('all-urls');
-    afterExcludedRequestDomainSet.delete('all-urls');
-
-    const addRules = [];
-    if (
-        allowEverywhere ||
-        afterRequestDomainSet.size !== 0 ||
-        afterExcludedRequestDomainSet.size !== 0
-    ) {
-        const rule0 = {
-            id: TRUSTED_DIRECTIVE_BASE_RULE_ID+0,
-            action: { type: 'allowAllRequests' },
-            condition: {
-                resourceTypes: [ 'main_frame' ],
-            },
-            priority: 100,
-        };
-        if ( afterRequestDomainSet.size !== 0 ) {
-            rule0.condition.requestDomains = Array.from(afterRequestDomainSet);
-        } else if ( afterExcludedRequestDomainSet.size !== 0 ) {
-            rule0.condition.excludedRequestDomains = Array.from(afterExcludedRequestDomainSet);
-        }
-        addRules.push(rule0);
-        dynamicRuleMap.set(TRUSTED_DIRECTIVE_BASE_RULE_ID+0, rule0);
-        // https://github.com/uBlockOrigin/uBOL-home/issues/114
-        const rule1 = {
-            id: TRUSTED_DIRECTIVE_BASE_RULE_ID+1,
-            action: { type: 'allow' },
-            condition: {
-                resourceTypes: [ 'script' ],
-            },
-            priority: 100,
-        };
-        if ( rule0.condition.requestDomains ) {
-            rule1.condition.initiatorDomains = rule0.condition.requestDomains.slice();
-        } else if ( rule0.condition.excludedRequestDomains ) {
-            rule1.condition.excludedInitiatorDomains = rule0.condition.excludedRequestDomains.slice();
-        }
-        addRules.push(rule1);
-        dynamicRuleMap.set(TRUSTED_DIRECTIVE_BASE_RULE_ID+1, rule1);
-    }
-
-    const updateOptions = { removeRuleIds };
-    if ( addRules.length ) {
-        updateOptions.addRules = addRules;
-    }
-    await dnr.updateDynamicRules(updateOptions);
-}
-
-/******************************************************************************/
-
 export async function getFilteringModeDetails() {
     const actualDetails = await readFilteringModeDetails();
     return {
