@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin - a comprehensive, efficient content blocker
     Copyright (C) 2015-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -19,13 +19,8 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/* global uDom */
-
-'use strict';
-
-/******************************************************************************/
-
-(( ) => {
+import { dom, qs$ } from './dom.js';
+import { i18n, i18n$ } from './i18n.js';
 
 /******************************************************************************/
 
@@ -50,56 +45,76 @@ let details = {};
 
     let lists;
     for ( const rawFilter in response ) {
-        if ( response.hasOwnProperty(rawFilter) ) {
+        if ( Object.prototype.hasOwnProperty.call(response, rawFilter) ) {
             lists = response[rawFilter];
             break;
         }
     }
 
-    if ( Array.isArray(lists) === false || lists.length === 0 ) { return; }
+    if ( Array.isArray(lists) === false || lists.length === 0 ) {
+        qs$('#whyex').style.setProperty('visibility', 'collapse');
+        return;
+    }
 
-    const parent = uDom.nodeFromSelector('#whyex > span:nth-of-type(2)');
+    const parent = qs$('#whyex > ul');
+    parent.firstElementChild.remove(); // remove placeholder element
     for ( const list of lists ) {
-        const listElem = document.querySelector('#templates .filterList')
-                                 .cloneNode(true);
-        const sourceElem = listElem.querySelector('.filterListSource');
+        const listElem = dom.clone('#templates .filterList');
+        const sourceElem = qs$(listElem, '.filterListSource');
         sourceElem.href += encodeURIComponent(list.assetKey);
-        sourceElem.textContent = list.title;
+        sourceElem.append(i18n.patchUnicodeFlags(list.title));
         if ( typeof list.supportURL === 'string' && list.supportURL !== '' ) {
-            const supportElem = listElem.querySelector('.filterListSupport');
-            supportElem.setAttribute('href', list.supportURL);
-            supportElem.classList.remove('hidden');
+            const supportElem = qs$(listElem, '.filterListSupport');
+            dom.attr(supportElem, 'href', list.supportURL);
+            dom.cl.remove(supportElem, 'hidden');
         }
         parent.appendChild(listElem);
     }
-    uDom.nodeFromId('whyex').style.removeProperty('display');
+    qs$('#whyex').style.removeProperty('visibility');
 })();
 
 /******************************************************************************/
 
-(( ) => {
-    const matches = /^(.*)\{\{hostname\}\}(.*)$/.exec(vAPI.i18n('docblockedProceed'));
-    if ( matches === null ) { return; }
-    const proceed = document.querySelector('#templates .proceed').cloneNode(true);
-    proceed.children[0].textContent = matches[1];
-    proceed.children[2].textContent = matches[2];
-    const hnOption = proceed.querySelector('.hn');
-    if ( details.hn !== details.dn ) {
-        hnOption.textContent = details.hn;
-        hnOption.setAttribute('value', details.hn);
-    } else {
-        hnOption.remove();
+const urlToFragment = raw => {
+    try {
+        const fragment = new DocumentFragment();
+        const url = new URL(raw);
+        const hn = url.hostname;
+        const i = raw.indexOf(hn);
+        const b = document.createElement('b');
+        b.append(hn);
+        fragment.append(raw.slice(0,i), b, raw.slice(i+hn.length));
+        return fragment;
+    } catch(_) {
     }
-    const dnOption = proceed.querySelector('.dn');
-    dnOption.textContent = details.dn;
-    dnOption.setAttribute('value', details.dn);
-    document.getElementById('proceed').append(proceed);
-})();
+    return raw;
+};
 
 /******************************************************************************/
 
-uDom.nodeFromSelector('#theURL > p > span:first-of-type').textContent = details.url;
-uDom.nodeFromId('why').textContent = details.fs;
+dom.clear('#theURL > p > span:first-of-type');
+qs$('#theURL > p > span:first-of-type').append(urlToFragment(details.url));
+dom.text('#why', details.fs);
+
+if ( typeof details.to === 'string' && details.to.length !== 0 ) {
+    const fragment = new DocumentFragment();
+    const text = i18n$('docblockedRedirectPrompt');
+    const linkPlaceholder = '{{url}}';
+    let pos = text.indexOf(linkPlaceholder);
+    if ( pos !== -1 ) {
+        const link = document.createElement('a');
+        link.href = details.to;
+        dom.cl.add(link, 'code');
+        link.append(urlToFragment(details.to)); 
+        fragment.append(
+            text.slice(0, pos),
+            link,
+            text.slice(pos + linkPlaceholder.length)
+        );
+        qs$('#urlskip').append(fragment);
+        dom.attr('#urlskip', 'hidden', null);
+    }
+}
 
 /******************************************************************************/
 
@@ -116,20 +131,21 @@ uDom.nodeFromId('why').textContent = details.fs;
             value = name;
             name = '';
         }
-        const li = document.createElement('li');
-        let span = document.createElement('span');
-        span.textContent = name;
+        const li = dom.create('li');
+        let span = dom.create('span');
+        dom.text(span, name);
         li.appendChild(span);
         if ( name !== '' && value !== '' ) {
             li.appendChild(document.createTextNode(' = '));
         }
-        span = document.createElement('span');
+        span = dom.create('span');
         if ( reURL.test(value) ) {
-            const a = document.createElement('a');
-            a.href = a.textContent = value;
+            const a = dom.create('a');
+            dom.attr(a, 'href', value);
+            dom.text(a, value);
             span.appendChild(a);
         } else {
-            span.textContent = value;
+            dom.text(span, value);
         }
         li.appendChild(span);
         return li;
@@ -149,14 +165,14 @@ uDom.nodeFromId('why').textContent = details.fs;
         if ( search === '' ) { return false; }
 
         url.search = '';
-        const li = liFromParam(vAPI.i18n('docblockedNoParamsPrompt'), url.href);
+        const li = liFromParam(i18n$('docblockedNoParamsPrompt'), url.href);
         parentNode.appendChild(li);
 
         const params = new self.URLSearchParams(search);
         for ( const [ name, value ] of params ) {
             const li = liFromParam(name, value);
             if ( depth < 2 && reURL.test(value) ) {
-                const ul = document.createElement('ul');
+                const ul = dom.create('ul');
                 renderParams(ul, value, depth + 1);
                 li.appendChild(ul);
             }
@@ -166,27 +182,22 @@ uDom.nodeFromId('why').textContent = details.fs;
         return true;
     };
 
-    if ( renderParams(uDom.nodeFromId('parsed'), details.url) === false ) {
+    if ( renderParams(qs$('#parsed'), details.url) === false ) {
         return;
     }
 
-    const toggler = document.querySelector('#toggleParse');
-    toggler.classList.remove('hidden');
+    dom.cl.remove('#toggleParse', 'hidden');
 
-    toggler.addEventListener('click', ( ) => {
-        const cl = uDom.nodeFromId('theURL').classList;
-        cl.toggle('collapsed');
+    dom.on('#toggleParse', 'click', ( ) => {
+        dom.cl.toggle('#theURL', 'collapsed');
         vAPI.localStorage.setItem(
             'document-blocked-expand-url',
-            (cl.contains('collapsed') === false).toString()
+            (dom.cl.has('#theURL', 'collapsed') === false).toString()
         );
     });
 
     vAPI.localStorage.getItemAsync('document-blocked-expand-url').then(value => {
-        uDom.nodeFromId('theURL').classList.toggle(
-            'collapsed',
-            value !== 'true' && value !== true
-        );
+        dom.cl.toggle('#theURL', 'collapsed', value !== 'true' && value !== true);
     });
 })();
 
@@ -195,31 +206,23 @@ uDom.nodeFromId('why').textContent = details.fs;
 // https://www.reddit.com/r/uBlockOrigin/comments/breeux/close_this_window_doesnt_work_on_firefox/
 
 if ( window.history.length > 1 ) {
-    uDom('#back').on(
-        'click',
-        ( ) => {
-            window.history.back();
-        }
-    );
-    uDom('#bye').css('display', 'none');
+    dom.on('#back', 'click', ( ) => {
+        window.history.back();
+    });
+    qs$('#bye').style.display = 'none';
 } else {
-    uDom('#bye').on(
-        'click',
-        ( ) => {
-            messaging.send('documentBlocked', {
-                what: 'closeThisTab',
-            });
-        }
-    );
-    uDom('#back').css('display', 'none');
+    dom.on('#bye', 'click', ( ) => {
+        messaging.send('documentBlocked', {
+            what: 'closeThisTab',
+        });
+    });
+    qs$('#back').style.display = 'none';
 }
 
 /******************************************************************************/
 
 const getTargetHostname = function() {
-    const elem = document.querySelector('#proceed select');
-    if ( elem === null ) { return details.hn; }
-    return elem.value;
+    return details.hn;
 };
 
 const proceedToURL = function() {
@@ -246,11 +249,18 @@ const proceedPermanent = async function() {
     proceedToURL();
 };
 
-uDom('#proceedTemporary').attr('href', details.url).on('click', proceedTemporary);
-uDom('#proceedPermanent').attr('href', details.url).on('click', proceedPermanent);
+dom.on('#disableWarning', 'change', ev => {
+    const checked = ev.target.checked;
+    dom.cl.toggle('[data-i18n="docblockedBack"]', 'disabled', checked);
+    dom.cl.toggle('[data-i18n="docblockedClose"]', 'disabled', checked);
+});
 
-/******************************************************************************/
-
-})();
+dom.on('#proceed', 'click', ( ) => {
+    if ( qs$('#disableWarning').checked ) {
+        proceedPermanent();
+    } else {
+        proceedTemporary();
+    }
+});
 
 /******************************************************************************/
